@@ -20,7 +20,7 @@ rmse[1] <- sqrt(mean((fit.bcart$ZZ.mean - ff$Ytrue)^2))
 
 ## dynamic treed constant model
 fit.dtc <- dynaTrees(X=X, y=Z, XX=XX, plotit=FALSE)
-rmse[2] <- sqrt(mean(apply(fit.dtc$mean, 1, mean) - ff$Ytrue)^2)
+rmse[2] <- sqrt(mean((apply(fit.dtc$mean, 1, mean) - ff$Ytrue)^2))
   
 ## MCMC treed linear model
 fit.btlm <- btlm(X=X, Z=Z, XX=XX, pred.n=FALSE, R=10,
@@ -29,7 +29,7 @@ rmse[3] <- sqrt(mean((fit.btlm$ZZ.mean - ff$Ytrue)^2))
 
 ## dynamic treed linear model
 fit.dtl <- dynaTrees(X=X, y=Z, XX=XX, model="linear", plotit=FALSE)
-rmse[4] <- sqrt(mean(apply(fit.dtl$mean, 1, mean) - ff$Ytrue)^2)
+rmse[4] <- sqrt(mean((apply(fit.dtl$mean, 1, mean) - ff$Ytrue)^2))
 
 ## MCMC GP
 fit.bgp <- bgp(X=X, Z=Z, XX=XX, pred.n=FALSE, BTE=c(2000,12000,2))
